/*
 * Copyright (C) 2009 by Eric Lambert <Eric.Lambert@sun.com>
 * Use and distribution licensed under the
 * GNU Lesser General Public License (LGPL) version 2.1.
 * See the COPYING file in the parent directory for full text.
 */

package org.gearman.worker;

import org.gearman.common.PacketType;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.util.ArrayList;
import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.gearman.Constants;
import org.gearman.common.GearmanPacket;
import org.gearman.client.GearmanJobEventListener;
import org.gearman.common.GearmanJobServerSession;
import org.gearman.common.GearmanServerRequest;

public class GearmanWorkerJobServerSession extends GearmanJobServerSession
        implements GearmanJobEventListener{

    private boolean isAwake;
    private ArrayList<GearmanPacket> assignedJobs;
    static final String DESCRIPTION_PREFIX = "GearmanWorkerJobServerSession";
    private final String DESCRIPTION;
    private static final Logger LOG = Logger.getLogger(
            Constants.GEARMAN_CLIENT_LOGGER_NAME);

    public GearmanWorkerJobServerSession(InetSocketAddress sa) {
        super(sa);
        isAwake = true;
        assignedJobs = new ArrayList<GearmanPacket>();
        DESCRIPTION = DESCRIPTION_PREFIX + ":" + Thread.currentThread().getId();
     }

    public void wakeSession() {
        LOG.log(Level.FINE,"Worker Session " + this + " is waking up.");
        isAwake = true;
    }

    public boolean isAwake() {
        return isAwake;
    }

    public void setToSleep() {
        LOG.log(Level.FINE,"Worker Session " + this + " is going to sleep.");
        isAwake = false;
    }

    public void addNewJob(GearmanPacket p) {
        if (p == null) {
            throw new IllegalArgumentException("Job Packet can not be null");
        }
        if ((!p.getPacketType().equals(PacketType.JOB_ASSIGN)) &&
                (!p.getPacketType().equals(PacketType.JOB_ASSIGN_UNIQ))) {
            throw new IllegalArgumentException("Job Packet " + p +
                    " was not a job assignment");
        }
        assignedJobs.add(p);
    }

    public Collection<GearmanPacket> getNewJobs() {
        Collection rc = (ArrayList)assignedJobs.clone();
        assignedJobs.clear();
        return rc;
    }

    public void handleGearmanJobEvent(GearmanPacket event) throws IllegalArgumentException {
        if (event == null) {
            throw new IllegalArgumentException("event can not be null");
        }

        LOG.log(Level.FINER,"Sesson " + this + " is sending request " +
                event + " to server ");
        GearmanServerRequest gsr = new GearmanServerRequest(event);
        this.sendRequestToServer(gsr);
        try {
            driveSessionIO();
        } catch (IOException ioe) {
            LOG.log(Level.WARNING,"Received an IOException while driving" +
                    " session: " + this,ioe);
        }
    }

}
