/*
 * Copyright (C) 2009 by Eric Lambert <Eric.Lambert@sun.com>
 * Use and distribution licensed under the
 * GNU Lesser General Public License (LGPL) version 2.1.
 * See the COPYING file in the parent directory for full text.
 */
package org.gearman.common;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.nio.ByteBuffer;
import java.util.Arrays;

import junit.framework.Assert;
import org.gearman.Constants;
import org.gearman.client.GearmanEchoResponseHandler;
import org.gearman.common.GearmanPacket;
import org.gearman.common.PacketMagic;
import org.gearman.common.PacketType;
import org.gearman.common.GearmanServerRequest.RequestState;

import org.gearman.common.GearmanPacket;
import org.junit.Test;

public class GearmanServerRequestTest {

    @Test
    public void ctorTest() {
        GearmanEchoResponseHandler handler = new GearmanEchoResponseHandler();
        GearmanPacket request = new Packet(PacketMagic.REQ, PacketType.ECHO_REQ,
                new byte[0]);
        GearmanPacket response = new Packet(PacketMagic.RES, PacketType.ECHO_RES,
                new byte[0]);
        GearmanServerRequest gsr = null;

        try {
            gsr = new GearmanServerRequest(handler, null);
            Assert.fail("Attempt to create GearmanServerRequest with a null" +
                    " request did not result an exception being raised");
        } catch (Throwable t) {
            Assert.assertTrue("Attempt to create GearmanServerRequest with" +
                    " a null request resulted in an unexpected exception" +
                    " being raised", t instanceof IllegalArgumentException);
        }

        try {
            gsr = new GearmanServerRequest(handler, response);
            Assert.fail("Attempt to create GearmanServerRequest with an" +
                    " invalid packet did not result an exception being raised");
        } catch (Throwable t) {
            Assert.assertTrue("Attempt to create GearmanServerRequest with" +
                    " an invalid packet resulted in an unexpected exception" +
                    " being raised", t instanceof IllegalArgumentException);
        }

        gsr = new GearmanServerRequest(handler, request);
        Assert.assertEquals("Incorrect state for newly requested" +
                " GearmanServerRequest.", RequestState.NEW, gsr.getState());
    }

    @Test
    public void toStringTest() {
        GearmanEchoResponseHandler handler = new GearmanEchoResponseHandler();
        GearmanPacket request = new Packet(PacketMagic.REQ,
                PacketType.ECHO_REQ, new byte[0]);
        GearmanServerRequest gsr = new GearmanServerRequest(handler, request);
        String s = gsr.toString();
        int firstDelim = s.indexOf(":");
        int lastDelim = s.lastIndexOf(":");
        Assert.assertTrue("String returned by toString (" + s + ") is not" +
                " in the expected format,", firstDelim > 0);
        Assert.assertTrue("String returned by toString (" + s + ") is not" +
                " in the expected format,", lastDelim != firstDelim);
        Assert.assertFalse("Unexpected suffix for String returned by toString",
                s.substring(lastDelim + 1).trim().equals(""));
        String expected  = GearmanServerRequest.DESCRIPTION_PREFIX + ":" + handler.toString();
        String actual =  s.substring(0, lastDelim);
        Assert.assertEquals("First two tokens of string returned by toString" +
                " do not match expected format.", expected,actual);
    }

    @Test
    public void getHandlerTest() {
        GearmanEchoResponseHandler handler = new GearmanEchoResponseHandler();
        GearmanPacket request = new Packet(PacketMagic.REQ,
                PacketType.ECHO_REQ, new byte[0]);
        GearmanServerRequest gsr = new GearmanServerRequest(handler, request);
        Assert.assertEquals("handler returned by getHandler does not match" +
                " handler passed into constructor.", handler, gsr.getHandler());
    }

    @Test
    public void driveRequestTest() {
        //TODO implement me
    }
}
